package sattose;

import java.io.BufferedReader;
import java.io.FileReader;
import java.io.IOException;
import java.io.StringReader;
import java.util.Iterator;
import java.util.List;
import java.util.Properties;
import java.util.Set;
import java.util.Vector;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import net.didion.jwnl.JWNL;
import net.didion.jwnl.JWNLException;
import net.didion.jwnl.data.IndexWord;
import net.didion.jwnl.data.IndexWordSet;
import net.didion.jwnl.data.POS;
import net.didion.jwnl.data.PointerType;
import net.didion.jwnl.data.PointerUtils;
import net.didion.jwnl.data.Synset;
import net.didion.jwnl.data.Word;
import net.didion.jwnl.data.list.PointerTargetNode;
import net.didion.jwnl.data.list.PointerTargetNodeList;
import net.didion.jwnl.data.relationship.RelationshipFinder;
import net.didion.jwnl.dictionary.Dictionary;
import edu.stanford.nlp.ling.CoreAnnotations.PartOfSpeechAnnotation;
import edu.stanford.nlp.ling.CoreAnnotations.TextAnnotation;
import edu.stanford.nlp.ling.CoreAnnotations.TokensAnnotation;
import edu.stanford.nlp.ling.CoreLabel;
import edu.stanford.nlp.parser.lexparser.LexicalizedParser;
import edu.stanford.nlp.pipeline.StanfordCoreNLP;
import edu.stanford.nlp.process.CoreLabelTokenFactory;
import edu.stanford.nlp.process.PTBTokenizer;
import edu.stanford.nlp.process.TokenizerFactory;
import edu.stanford.nlp.trees.Tree;
import edu.stanford.nlp.util.CoreMap;

public class NLPUtil {

    private static NLPUtil util = null;

    public static int minCharsForTerms = 3;

    // Stanford CoreNLP:
    private static StanfordCoreNLP pipeline;
    private static LexicalizedParser lp;
    private static TokenizerFactory<CoreLabel> tokenizerFactory;
    private static Properties props;

    // WordNet:
    private static Dictionary wordNetDictionary = null;

    public static Dictionary getWordNetDictionary() throws JWNLException {
	NLPUtil.getInstance();
	return NLPUtil.wordNetDictionary;
    }

    public static NLPUtil getInstance() throws JWNLException {
	if (NLPUtil.util == null) {
	    NLPUtil.util = new NLPUtil();
	}
	return NLPUtil.util;
    }

    private NLPUtil() throws JWNLException {
	NLPUtil.lp = LexicalizedParser
		.loadModel("edu/stanford/nlp/models/lexparser/englishPCFG.ser.gz");
	NLPUtil.tokenizerFactory = PTBTokenizer.factory(
		new CoreLabelTokenFactory(), "");
	NLPUtil.props = new Properties();
	NLPUtil.props.put("annotators", "tokenize, ssplit, pos, lemma");
	NLPUtil.pipeline = new StanfordCoreNLP(NLPUtil.props);

	// WordNet
	JWNL.initialize(WordnetConfiguration.getInputStream());
	NLPUtil.wordNetDictionary = Dictionary.getInstance();

	// load English and Java stop words lists:
	NLPUtil.programmingKeywordsStopWordsPattern = NLPUtil
		.loadFromFile(ProjectConfiguration.STOPWORDS_JAVA_KEYWORDS);
	NLPUtil.programmingSpecificStopWordsPattern = NLPUtil
		.loadFromFile(ProjectConfiguration.STOPWORDS_JAVA_OTHERS);
	NLPUtil.generalStopWordsPattern = NLPUtil
		.loadFromFile(ProjectConfiguration.STOPWORDS_ENGLISH);

    }

    private static Pattern loadFromFile(final String fileName) {
	String result = "\\b(";
	try {
	    final BufferedReader iLOF = new BufferedReader(new FileReader(
		    fileName));
	    try {
		String word = null;
		while ((word = iLOF.readLine()) != null) {
		    result = result + word + "|";
		}
	    } finally {
		iLOF.close();
	    }
	} catch (final IOException e) {
	    e.printStackTrace();
	}

	result = result.substring(0, result.length() - 1) + ")\\b";

	System.out.println(result);

	return Pattern.compile(result, Pattern.CASE_INSENSITIVE);

    }

    private static String addAllTerms(final List<String> identifierSplit) {
	String result = "";
	for (final String t : identifierSplit) {
	    result = result + t + " ";
	}
	return result;
    }

    public static String applySplitOnlyTemplate(
	    final List<String> identifierSplit) {
	final String result = NLPUtil.addAllTerms(identifierSplit);
	if (result.endsWith(" ")) {
	    return result.substring(0, result.length() - 1);
	}
	return result.toLowerCase();
    }

    public static String identifierToSentence(final String identifier) {
	final List<String> identifierSplit = Splitter
		.splitIdentifier(identifier);
	final String sentence = NLPUtil.applySplitOnlyTemplate(identifierSplit)
		.toLowerCase();
	return sentence;
    }

    public static String getLemma(final CoreMap sentence, final String word) {
	for (final CoreLabel toto : sentence.get(TokensAnnotation.class)) {
	    if (toto.originalText().equalsIgnoreCase(word)) {
		return toto.lemma();
	    }
	}
	return null;
    }

    public static List<String> tokenize(final String sentence) {
	final List<String> result = new Vector<String>();
	NLPUtil.tokenizerFactory.toString();
	final List<CoreLabel> tokenizedSentence = NLPUtil.tokenizerFactory
		.getTokenizer(new StringReader(sentence)).tokenize();
	for (final CoreLabel cl : tokenizedSentence) {
	    result.add(cl.toString());
	}
	return result;
    }

    public static String getPOS(final Tree tree, final String term) {
	final Iterator<CoreLabel> it = tree.taggedLabeledYield().iterator();
	CoreLabel leaf = null;
	String word;
	while (it.hasNext()) {
	    leaf = it.next();
	    word = leaf.get(TextAnnotation.class);
	    if (word.compareTo(term) == 0) {
		return leaf.get(PartOfSpeechAnnotation.class);
	    }
	}
	return null;
    }

    public static String getPOS(final String template, final String term) {
	final Iterator<CoreLabel> it = NLPUtil.getTree(template)
		.taggedLabeledYield().iterator();
	CoreLabel leaf = null;
	String word;
	while (it.hasNext()) {
	    leaf = it.next();
	    word = leaf.get(TextAnnotation.class);
	    if (word.compareTo(term) == 0) {
		return leaf.get(PartOfSpeechAnnotation.class);
	    }
	}
	return null;
    }

    public static boolean hasPOS(final String template, final String pos) {
	final Iterator<CoreLabel> it = NLPUtil.getTree(template)
		.taggedLabeledYield().iterator();
	CoreLabel leaf = null;
	while (it.hasNext()) {
	    leaf = it.next();
	    if (leaf.get(PartOfSpeechAnnotation.class).equalsIgnoreCase(pos)) {
		return true;
	    }
	}
	return false;
    }

    public static boolean startsWithVerb(final String template) {
	final Iterator<CoreLabel> it = NLPUtil.getTree(template)
		.taggedLabeledYield().iterator();
	final CoreLabel leaf = it.next();
	if (leaf.get(PartOfSpeechAnnotation.class).startsWith("VB")) {
	    return true;
	}
	return false;
    }

    public static void annotate(
	    final edu.stanford.nlp.pipeline.Annotation document) {
	NLPUtil.pipeline.annotate(document);
    }

    public static Tree getTree(final String template) {
	final List<CoreLabel> rawWordsOld = NLPUtil.tokenizerFactory
		.getTokenizer(new StringReader(template)).tokenize();
	return NLPUtil.lp.apply(rawWordsOld);
    }

    public static boolean containsAntonyms(final IndexWord iwStart,
	    final IndexWord iwEnd, final boolean indirect,
	    final boolean extended, final int depth) throws JWNLException {
	Synset startSyn;
	Synset endSyn;
	for (int i = 1; i <= iwStart.getSenseCount(); i++) {
	    startSyn = iwStart.getSense(i);
	    final List allAntos = new Vector();

	    allAntos.addAll(PointerUtils.getInstance().getAntonyms(startSyn));

	    if (indirect) {
		allAntos.addAll(PointerUtils.getInstance()
			.getIndirectAntonyms(startSyn, depth).toList());
	    }
	    if (extended) {
		allAntos.addAll(PointerUtils.getInstance()
			.getExtendedAntonyms(startSyn, depth).toList());
	    }

	    for (int j = 1; j <= iwEnd.getSenseCount(); j++) {
		endSyn = iwEnd.getSense(j);

		for (final Object nlist : allAntos) {
		    if (nlist instanceof PointerTargetNodeList) {
			for (int ind = 0; ind < ((PointerTargetNodeList) nlist)
				.size(); ind++) {

			    final PointerTargetNode node = (PointerTargetNode) ((PointerTargetNodeList) nlist)
				    .get(ind);
			    for (final Word meaning : node.getSynset()
				    .getWords()) {
				if (NLPUtil
					.contains(endSyn, meaning.getLemma())) {
				    return true;
				}

			    }
			}
		    } else {
			for (final Word meaning : ((PointerTargetNode) nlist)
				.getSynset().getWords()) {
			    if (NLPUtil.contains(endSyn, meaning.getLemma())) {
				return true;
			    }
			}
		    }
		}

		if (
		// works for verbs, adverbs, nouns:
		RelationshipFinder
			.getInstance()
			.findRelationships(startSyn, endSyn,
				PointerType.ANTONYM).size() > 0) {
		    return true;
		}

	    }
	}
	return false;
    }

    public static boolean checkForSimilar(final IndexWord iwStart,
	    final IndexWord iwEnd) throws JWNLException {
	Synset startSyn;
	Synset endSyn;
	for (int i = 1; i <= iwStart.getSenseCount(); i++) {
	    startSyn = iwStart.getSense(i);

	    for (int j = 1; j <= iwEnd.getSenseCount(); j++) {
		endSyn = iwEnd.getSense(j);

		// Check for SIMILAR_TO:
		if (RelationshipFinder
			.getInstance()
			.findRelationships(startSyn, endSyn,
				PointerType.SIMILAR_TO).size() > 0) {
		    return true;
		}

	    }
	}
	return false;
    }

    private static boolean contains(final Synset synset, final String lemma) {
	for (final Word w : synset.getWords()) {
	    if (w.getLemma().toLowerCase()
		    .equalsIgnoreCase(lemma.toLowerCase())) {
		return true;
	    }
	}
	return false;
    }

    private static Pattern programmingKeywordsStopWordsPattern;
    private static Pattern programmingSpecificStopWordsPattern;
    private static Pattern generalStopWordsPattern;

    public static String removeStopWords(final String initialString,
	    final boolean applyProgrammingKeywordsStopWords,
	    final boolean applyProgrammingSpecificStopWords,
	    final boolean applyGeneralStopWords) {
	String result = initialString;
	if (applyProgrammingKeywordsStopWords) {
	    final Matcher matcher = NLPUtil.programmingKeywordsStopWordsPattern
		    .matcher(result);
	    result = matcher.replaceAll(" ");
	}
	if (applyProgrammingSpecificStopWords) {
	    final Matcher matcher = NLPUtil.programmingSpecificStopWordsPattern
		    .matcher(result);
	    result = matcher.replaceAll(" ");
	}
	if (applyGeneralStopWords) {
	    final Matcher matcher = NLPUtil.generalStopWordsPattern
		    .matcher(result);
	    result = matcher.replaceAll(" ");
	}

	return result;
    }

    public static List<String> getAntonymPairs(
	    final List<String> termsInIdentifier,
	    final List<String> termsInComments) throws JWNLException {
	final List<String> antonymPairs = new Vector<String>();
	for (final String termInIdentifier : termsInIdentifier) {
	    final IndexWordSet termInIdentifierIWS = NLPUtil
		    .getWordNetDictionary().lookupAllIndexWords(
			    termInIdentifier);
	    final Set termInIdentifierPOSSet = termInIdentifierIWS
		    .getValidPOSSet();
	    for (final String termInComments : termsInComments) {

		final IndexWordSet termInCommentsIWS = NLPUtil
			.getWordNetDictionary().lookupAllIndexWords(
				termInComments);

		final Set termInCommentsPOSSet = termInCommentsIWS
			.getValidPOSSet();
		IndexWord termInIdentifierIW, termInCommentsIW;

		for (final Object currentPos : termInIdentifierPOSSet) {
		    if (termInCommentsPOSSet.contains(currentPos)) {

			termInIdentifierIW = NLPUtil.getWordNetDictionary()
				.lookupIndexWord((POS) currentPos,
					termInIdentifierIWS.getLemma());
			termInCommentsIW = NLPUtil.getWordNetDictionary()
				.lookupIndexWord((POS) currentPos,
					termInCommentsIWS.getLemma());
			if (NLPUtil.containsAntonyms(termInIdentifierIW,
				termInCommentsIW, true, true, 1)
				&& !NLPUtil.checkForSimilar(termInIdentifierIW,
					termInCommentsIW)) {
			    antonymPairs.add(termInIdentifier + "_"
				    + termInComments + "_" + currentPos);

			}

		    }
		}

	    }
	}
	return antonymPairs;
    }

}
