package sattose;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

import net.didion.jwnl.JWNLException;
import nu.xom.Builder;
import nu.xom.Document;
import nu.xom.Element;
import nu.xom.Node;
import nu.xom.Nodes;
import nu.xom.ParentNode;
import nu.xom.ParsingException;
import nu.xom.ValidityException;
import nu.xom.XPathContext;

public class SrcMLParser {

    public static final XPathContext namespace = new XPathContext("srcml",
	    "http://www.srcML.org/srcML/src");
    List<String> fileList = new ArrayList<String>();
    Document doc = null;

    public static XPathContext getNamespace() {
	return SrcMLParser.namespace;
    }

    public List<String> getFileList() {
	return this.fileList;
    }

    public Document getDoc() {
	return this.doc;
    }

    // Method that gets all the xml files that srcML produced are returns their
    // file path in a list
    public List<String> ititializeParser(final String xmlFile) {

	// Path to test output folder containing xml parsed by srcML
	final Builder builder = new Builder();

	try {
	    this.doc = builder.build(xmlFile);
	} catch (final ValidityException e) {
	    e.printStackTrace();
	} catch (final ParsingException e) {
	    e.printStackTrace();
	} catch (final IOException e) {
	    e.printStackTrace();
	}

	final Nodes fileNames = this.doc.query("//srcml:unit/@filename",
		SrcMLParser.namespace);
	for (int fileNamesNum = 0; fileNamesNum < fileNames.size(); fileNamesNum++) {
	    System.out.println("\tFile name: "
		    + fileNames.get(fileNamesNum).getValue());
	    this.fileList.add(fileNames.get(fileNamesNum).getValue());
	}

	System.out.println("Number of files:" + fileNames.size());

	return this.fileList;
    }

    public void processUnit(final String fileName) {

	System.out.println("Processing: " + fileName);
	final Node unitNode = this.getUnit(fileName);
	// Get root node
	final Element root = this.doc.getRootElement();
	System.out.println("Node: " + unitNode.getValue());

	final Nodes attributes = this.getAttributes(unitNode);
	System.out.println("Number of Attributes:  " + attributes.size());
	this.printIdentifiers(this.getAttributeIdentifiers(attributes));
	for (int nodesNum = 0; nodesNum < attributes.size(); nodesNum++) {
	    final Node currentAttribute = attributes.get(nodesNum);
	    System.out.println(this
		    .getAttributeIdentifierAndComment(currentAttribute));
	    try {
		final List<String> termsInIdentifier = Splitter
			.splitIdentifier(this
				.getAttributeIdentifier(currentAttribute));
		final List<String> termsInComments = this
			.extractTerms(this.nodesToStrings(this
				.getPreceedingCommentsForAttribute(currentAttribute)));
		System.out.println("ANTONYM PAIRS:"
			+ NLPUtil.getAntonymPairs(termsInIdentifier,
				termsInComments));
	    } catch (final JWNLException e) {
		e.printStackTrace();
	    }
	}

	final Nodes methods = this.getMethods(unitNode);
	System.out.println("Number of Methods:  " + methods.size());
	this.printIdentifiers(this.getMethodIdentifiers(methods));
	for (int nodesNum = 0; nodesNum < methods.size(); nodesNum++) {
	    System.out.println(this.getMethodIdentifierAndComment(methods
		    .get(nodesNum)));
	    if (NLPUtil.startsWithVerb(NLPUtil.applySplitOnlyTemplate(Splitter
		    .splitIdentifier(this.getMethodIdentifier(methods
			    .get(nodesNum)))))) {
		System.out.println("STARTS WITH VERB: true " + methods.size());
	    }
	}

    }

    private List<String> extractTerms(final List<String> comments) {
	final List<String> terms = new Vector<String>();
	for (final String comment : comments) {
	    terms.addAll(NLPUtil.tokenize(comment));
	}
	return terms;
    }

    public void printIdentifiers(final List<String> identifiers) {
	for (final String id : identifiers) {
	    System.out.println("\t IDENTIFIER: " + id);
	}
    }

    public List<String> getClassIdentifiers(final Nodes someNodes) {
	final List<String> identifiers = new Vector<String>();
	Node aNode;
	for (int nodesNum = 0; nodesNum < someNodes.size(); nodesNum++) {
	    aNode = someNodes.get(nodesNum);
	    identifiers.add(SrcMLParser.getClassIdentifier(aNode));
	}
	return identifiers;
    }

    public List<String> getAttributeIdentifiers(final Nodes someNodes) {
	final List<String> identifiers = new Vector<String>();
	Node aNode;
	for (int nodesNum = 0; nodesNum < someNodes.size(); nodesNum++) {
	    aNode = someNodes.get(nodesNum);
	    identifiers.add(this.getAttributeIdentifier(aNode));
	}
	return identifiers;
    }

    public List<String> getMethodIdentifiers(final Nodes someNodes) {
	final List<String> identifiers = new Vector<String>();
	Node aNode;
	for (int nodesNum = 0; nodesNum < someNodes.size(); nodesNum++) {
	    aNode = someNodes.get(nodesNum);
	    identifiers.add(this.getMethodIdentifier(aNode));
	}
	return identifiers;
    }

    public Node getUnit(final String fileName) {
	final Nodes unitNode = this.doc.query("//srcml:unit[@filename='"
		+ fileName + "']", SrcMLParser.namespace);
	return unitNode.get(0);
    }

    public Nodes getClassNodes(final Node node) {
	final Nodes classNodes = node.query(".//srcml:class",
		SrcMLParser.namespace);
	return classNodes;
    }

    public Nodes getMethods(final Node node) {
	final Nodes methNodes = node
		.query(".//srcml:function[../parent::srcml:class/srcml:block] | "
			+ ".//srcml:function_decl[../parent::srcml:class/srcml:block]",
			SrcMLParser.namespace);
	return methNodes;
    }

    public Nodes getAttributes(final Node node) {
	final Nodes attrNodes = node
		.query(".//srcml:decl_stmt[../parent::srcml:class/srcml:block]/srcml:decl",
			SrcMLParser.namespace);
	return attrNodes;
    }

    public static String getClassIdentifier(final Node node) {
	return node.query(".//srcml:name", SrcMLParser.namespace).get(0)
		.getValue();
    }

    public String getAttributeIdentifier(final Node node) {
	return node
		.query(".//srcml:name[./parent::srcml:decl]",
			SrcMLParser.namespace).get(0).getValue();
    }

    public String getMethodIdentifier(final Node node) {
	return node.query("./srcml:name", SrcMLParser.namespace).get(0)
		.getValue();
    }

    public Nodes getPreceedingCommentsForMethod(final Node aNode) {
	final ParentNode parent = aNode.getParent();

	// Find the last preceeding sibling that is not a comment and get all
	// comments that are following it but that are preceeding siblings of
	// the current node.
	final Nodes preceedingSiblingsNotComments = aNode.query(
		"preceding-sibling::*[not(self::srcml:comment)]",
		SrcMLParser.namespace);
	if (preceedingSiblingsNotComments.size() > 0) {
	    final Node lastSiblingNotAComment = preceedingSiblingsNotComments
		    .get(preceedingSiblingsNotComments.size() - 1);
	    final int indexOfLastSiblingNotAComment = parent
		    .indexOf(lastSiblingNotAComment);

	    final Nodes preceedingSiblingsComments = aNode.query(
		    "preceding-sibling::srcml:comment", SrcMLParser.namespace);

	    final Nodes result = new Nodes();
	    int indexCurrentComment = preceedingSiblingsComments.size() - 1;
	    while (indexCurrentComment > 0) {
		if (parent.indexOf(preceedingSiblingsComments
			.get(indexCurrentComment)) > indexOfLastSiblingNotAComment) {
		    result.append(preceedingSiblingsComments
			    .get(indexCurrentComment));
		}
		indexCurrentComment--;
	    }
	    return result;
	} else {
	    return aNode.query("preceding-sibling::srcml:comment",
		    SrcMLParser.namespace);
	}
    }

    public List<String> nodesToStrings(final Nodes someNodes) {
	final List<String> result = new Vector<String>();
	for (int i = 0; i < someNodes.size(); i++) {
	    result.add(someNodes.get(i).getValue());
	}
	return result;
    }

    // FIXME: The current implementation collects comments that do not
    // apply to the attribute passed as a parameter. Fix it.
    public Nodes getPreceedingCommentsForAttribute(final Node aNode) {
	return aNode.query("../preceding-sibling::srcml:comment",
		SrcMLParser.namespace);
    }

    public String getMethodIdentifierAndComment(final Node node) {
	final List<String> comments = this.nodesToStrings(this
		.getPreceedingCommentsForMethod(node));
	final String identifier = this.getMethodIdentifier(node);
	return "IDENTIFIER: " + identifier + "\n" + "COMMENTS:"
		+ comments.toString();

    }

    public String getAttributeIdentifierAndComment(final Node node) {
	final List<String> comments = this.nodesToStrings(this
		.getPreceedingCommentsForAttribute(node));
	final String identifier = this.getAttributeIdentifier(node);
	return "IDENTIFIER: " + identifier + "\n" + "COMMENTS:"
		+ comments.toString();

    }

}
