package sattose.tests;

import junit.framework.TestCase;
import net.didion.jwnl.JWNLException;
import net.didion.jwnl.data.IndexWord;
import net.didion.jwnl.data.POS;
import nu.xom.Node;
import sattose.NLPUtil;
import sattose.ProjectConfiguration;
import sattose.SrcMLParser;

public class SaTToSETest extends TestCase {

    SrcMLParser parser = null;
    String projectFolder = ProjectConfiguration.projectFolder;

    public SaTToSETest(final String name) {
	super(name);
    }

    protected void setUp() throws Exception {
	super.setUp();
	if (this.parser == null) {
	    this.parser = new SrcMLParser();
	    this.parser.ititializeParser("/" + this.projectFolder
		    + "src/sattose/tests/inputs-srcml.xml");
	}
    }

    protected void tearDown() throws Exception {
	super.tearDown();
    }

    public void testItitializeParser() {
	TestCase.assertEquals(1, this.parser.getFileList().size());
    }

    public void testGetUnit() {
	TestCase.assertNotNull(this.parser.getUnit(this.projectFolder
		+ "src/sattose/tests/inputs/Class1.java"));
    }

    public void testGetClassNodes() {

	TestCase.assertEquals(
		3,
		this.parser.getClassNodes(
			this.parser.getUnit(this.projectFolder
				+ "src/sattose/tests/inputs/Class1.java"))
			.size());
    }

    public void testGetAttributes() {
	// get the unit node that are declared in the file
	final Node unitNode = this.parser.getUnit(this.projectFolder
		+ "src/sattose/tests/inputs/Class1.java");

	// attribute1_visible, attribute1OfClass3, and attribute2OfClass3
	TestCase.assertEquals(3, this.parser.getAttributes(unitNode).size());

    }

    public void testGetMethods() {
	// get the unit node that are declared in the file
	final Node unitNode = this.parser.getUnit(this.projectFolder
		+ "src/sattose/tests/inputs/Class1.java");

	// method1, method2, and method1OfClass2
	TestCase.assertEquals(3, this.parser.getMethods(unitNode).size());

    }

    public void testGetPreceedingComments() {
	// get the unit node that are declared in the file
	final Node unitNode = this.parser.getUnit(this.projectFolder
		+ "src/sattose/tests/inputs/Class1.java");

	/*
	 * ATTRIBUTES:
	 */
	// attribute1_visible
	final Node attribute1_visible = this.parser.getAttributes(unitNode)
		.get(0);
	TestCase.assertEquals("attribute1_visible",
		this.parser.getAttributeIdentifier(attribute1_visible));
	// has 3 comments
	TestCase.assertEquals(
		3,
		this.parser.getPreceedingCommentsForAttribute(
			attribute1_visible).size());

	// attribute1OfClass3
	final Node attribute1OfClass3 = this.parser.getAttributes(unitNode)
		.get(1);
	TestCase.assertEquals("attribute1OfClass3",
		this.parser.getAttributeIdentifier(attribute1OfClass3));
	// has 1 comment
	TestCase.assertEquals(
		1,
		this.parser.getPreceedingCommentsForAttribute(
			attribute1OfClass3).size());

	// attribute2OfClass3
	final Node attribute2OfClass3 = this.parser.getAttributes(unitNode)
		.get(2);
	TestCase.assertEquals("attribute2OfClass3",
		this.parser.getAttributeIdentifier(attribute2OfClass3));
	// has 1 comment
	TestCase.assertEquals(
		1,
		this.parser.getPreceedingCommentsForAttribute(
			attribute2OfClass3).size());

	/*
	 * METHODS:
	 */
	// method1
	final Node method1 = this.parser.getMethods(unitNode).get(0);
	TestCase.assertEquals("method1",
		this.parser.getMethodIdentifier(method1));
	// has 1 comment
	TestCase.assertEquals(1,
		this.parser.getPreceedingCommentsForMethod(method1).size());

	// method1OfClass2
	final Node method1OfClass2 = this.parser.getMethods(unitNode).get(1);
	TestCase.assertEquals("method1OfClass2",
		this.parser.getMethodIdentifier(method1OfClass2));
	// has 2 comments
	TestCase.assertEquals(2,
		this.parser.getPreceedingCommentsForMethod(method1OfClass2)
			.size());

	// method2
	final Node doSomething = this.parser.getMethods(unitNode).get(2);
	TestCase.assertEquals("doSomething",
		this.parser.getMethodIdentifier(doSomething));
	// has 2 comments
	TestCase.assertEquals(2,
		this.parser.getPreceedingCommentsForMethod(doSomething).size());

    }

    public void testContainsForAntonyms() {
	IndexWord termInIdentifierIW, termInCommentsIW;
	try {
	    // visible and hidden:
	    termInIdentifierIW = NLPUtil.getWordNetDictionary()
		    .lookupIndexWord(POS.ADJECTIVE, "visible");
	    termInCommentsIW = NLPUtil.getWordNetDictionary().lookupIndexWord(
		    POS.ADJECTIVE, "invisible");
	    TestCase.assertTrue(NLPUtil.containsAntonyms(termInIdentifierIW,
		    termInCommentsIW, false, false, 1));

	    termInIdentifierIW = NLPUtil.getWordNetDictionary()
		    .lookupIndexWord(POS.ADJECTIVE, "visible");
	    termInCommentsIW = NLPUtil.getWordNetDictionary().lookupIndexWord(
		    POS.ADJECTIVE, "unavailable");
	    TestCase.assertTrue(NLPUtil.containsAntonyms(termInIdentifierIW,
		    termInCommentsIW,
		    // indirect (to get 'unavailable', indirect
		    // antonym via 'available')?
		    true,
		    // extended (to get 'hidden'):
		    false, 1));

	    termInIdentifierIW = NLPUtil.getWordNetDictionary()
		    .lookupIndexWord(POS.ADJECTIVE, "visible");
	    termInCommentsIW = NLPUtil.getWordNetDictionary().lookupIndexWord(
		    POS.ADJECTIVE, "hidden");
	    TestCase.assertTrue(NLPUtil.containsAntonyms(termInIdentifierIW,
		    termInCommentsIW, false,
		    // extended (to get 'hidden'):
		    true, 1));

	} catch (final JWNLException e) {
	    e.printStackTrace();
	}

    }

}
